﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Text;
using System.Diagnostics;
using System.Threading.Tasks;

namespace PickMe2021
{
    static class DirectoryTraverse
    {
        //https://stackoverflow.com/a/12262552

        [System.Runtime.InteropServices.DllImport("shell32.dll", ExactSpelling = true)]
        public static extern void ILFree(IntPtr pidlList);

        [System.Runtime.InteropServices.DllImport("shell32.dll", CharSet = System.Runtime.InteropServices.CharSet.Unicode, ExactSpelling = true)]
        public static extern IntPtr ILCreateFromPathW(string pszPath);

        [System.Runtime.InteropServices.DllImport("shell32.dll", ExactSpelling = true)]
        public static extern int SHOpenFolderAndSelectItems(IntPtr pidlList, uint cild, IntPtr children, uint dwFlags);

        private static void SelectInFileExplorer(string fullPath)
        {
            if (string.IsNullOrEmpty(fullPath))
                throw new ArgumentNullException("fullPath");

            fullPath = Path.GetFullPath(fullPath);

            IntPtr pidlList = ILCreateFromPathW(fullPath);
            if (pidlList != IntPtr.Zero)
                try
                {
                    // Open parent folder and select item
                    System.Runtime.InteropServices.Marshal.ThrowExceptionForHR(SHOpenFolderAndSelectItems(pidlList, 0, IntPtr.Zero, 0));
                }
                finally
                {
                    ILFree(pidlList);
                }
        }

        public static IEnumerable<FileInfo> TraverseDirectory(DirectoryInfo dir, IEnumerable<string> allowedExtensions = null)
        {
            IEnumerable<FileInfo> files = dir.GetFiles();
            IEnumerable<DirectoryInfo> subdirs = dir.GetDirectories();

            if (allowedExtensions != null)
            {
                files = files.Where(x => allowedExtensions.Contains(x.Extension.ToLower()));
            }

            foreach (var subdir in subdirs)
            {
                files = files.Concat(TraverseDirectory(subdir, allowedExtensions));
            }

            return files;
        }

        public static void OpenExplorerToFile(FileInfo fileToOpen)
        {
            var path = Path.GetFullPath(fileToOpen.FullName);
            SelectInFileExplorer(path);
        }
    }
}
